/** \file
    \brief Funkcje o charakterze narzdziowym (czyli mietnik)
    \date October 2006
*/

#include <vcl.h>
#pragma hdrstop

//---------------------------------------------------------------------------
#pragma package(smart_init)

#include <string.h>
#include <dir.h>
#include "utils.h"

/** \brief Utwrz SIP URI z podanego tekstu - dodaj ewentualny "sip:"
    \param tekst do przeksztacenia
    \return SIP URI w formie <"sip:"+tekst> lub <tekst>
*/
AnsiString MakeURI(AnsiString tekst)
{
    if(strncmp(tekst.c_str(), "sip:", 4))
    {
        return (AnsiString)"sip:" + tekst;
    }
    else
        return tekst;
}


/** \brief Utwrz SIP URI dla poczenia wychodzcego dodajc ew. "sip:" i "@serwer"
    \param tekst do przeksztacenia
    \return SIP URI w formie <"sip:"+tekst> lub <tekst>
*/
AnsiString MakeOutURI(AnsiString tekst, AnsiString serwer)
{
    AnsiString strValue, strValue2;
    if(strncmp(tekst.c_str(), "sip:", 4) && tekst.Length()>0)
    {
        strValue = (AnsiString)"sip:" + tekst;
        if(!strchr(strValue.c_str(), (int)'@'))   // ewentualne dodanie ...@serwer
        {
            if(strncmp(serwer.c_str(), "sip:", 4))  //inny format nazwy serwera ni sip:...
                strValue2 = (AnsiString)"@" + serwer;
            else    // registrar jest w formie sip:..., trzeba usun nazw protokou
            {
                strValue2 = (AnsiString)"@" + serwer.SubString(5, serwer.Length()-4);
            }
            strValue += strValue2;
        }
    }
    else if( tekst.Length()>0 )
        strValue = tekst;


    return strValue;
}

/** Escape apostrophe (double it) - Firebird escape
*/
AnsiString ReplaceQuotes (AnsiString text)
{
    text = text.Trim();
//	return StringReplace(text, "'", "\"", TReplaceFlags() << rfReplaceAll);
	return StringReplace(text, "'", "''", TReplaceFlags() << rfReplaceAll);
}


typedef BOOL (WINAPI *P_GDFSE)(LPCTSTR, PULARGE_INTEGER,
                              PULARGE_INTEGER, PULARGE_INTEGER);
/** Get amount of free disk space in MBytes
*/
int GetPartitionFreeSpace(char *pszDrive)
{
    //[C++ Fatal Error] utils.cpp(149): F1001 Internal code generator error
    //po wczeniu codeguarda
#ifndef __CODEGUARD__

  BOOL  fResult = false;

  char  szDrive[5];

  DWORD dwSectPerClust,
        dwBytesPerSect,
        dwFreeClusters,
        dwTotalClusters;

  P_GDFSE pGetDiskFreeSpaceEx = NULL;

  unsigned __int64 i64FreeBytesToCaller,
                   i64TotalBytes,
                   i64FreeBytes;

  /*
     Command line parsing.

     If the drive is a drive letter and not a UNC path, append a
     trailing backslash to the drive letter and colon.  This is
     required on Windows 95 and 98.
  */ 

  if (pszDrive[1] == ':')
  {
     szDrive[0] = pszDrive[0];
     szDrive[1] = ':';
     szDrive[2] = '\\';
     szDrive[3] = '\\';
     szDrive[4] = '\0';

     pszDrive = szDrive;
  }

  /*
     Use GetDiskFreeSpaceEx if available; otherwise, use
     GetDiskFreeSpace.

     Note: Since GetDiskFreeSpaceEx is not in Windows 95 Retail, we
     dynamically link to it and only call it if it is present.  We 
     don't need to call LoadLibrary on KERNEL32.DLL because it is 
     already loaded into every Win32 process's address space.
  */
  pGetDiskFreeSpaceEx = (P_GDFSE)GetProcAddress (
                           GetModuleHandle ("kernel32.dll"),
                                            "GetDiskFreeSpaceExA");
  if (pGetDiskFreeSpaceEx)
  {
     fResult = pGetDiskFreeSpaceEx (pszDrive,
                             (PULARGE_INTEGER)&i64FreeBytesToCaller,
                             (PULARGE_INTEGER)&i64TotalBytes,
                             (PULARGE_INTEGER)&i64FreeBytes);
     if (fResult)
     {
        return (i64FreeBytes / (1024*1024));
     }
  }
  else
  {
     fResult = GetDiskFreeSpace (pszDrive, 
                                 &dwSectPerClust,
                                 &dwBytesPerSect, 
                                 &dwFreeClusters,
                                 &dwTotalClusters);
     if (fResult)
     {
        /* force 64-bit math */
        i64FreeBytes = (__int64)dwFreeClusters * dwSectPerClust *
                          dwBytesPerSect;
        return (i64FreeBytes / (1024*1024));
     }
  }

#endif  //#ifndef __CODEGUARD__
  return 9999999;
}

/** Get directory size including subdirectories
*/
int GetDirSize(AnsiString path, int iAttributes)
{
    iAttributes |= faDirectory;
    TSearchRec sr;
    unsigned int iFolderSize = 0;
    if (FindFirst(path, iAttributes, sr) == 0)
    {
        do
        {
            iFolderSize += sr.Size;
            // drill into subdirs
            if (sr.Attr & faDirectory) {
              // Do not drill into "." or ".."
              if (sr.Name != "." && sr.Name != "..") {
                if(path.Length() + sr.Name.Length() < MAXPATH-1)
                {
	                AnsiString drillPath = appendDirectory(path, sr.Name);
    	            iFolderSize += GetDirSize(drillPath, iAttributes);
                }
              }
          }
        } while (FindNext(sr) == 0);
        FindClose(sr);
    }
	return iFolderSize;
}


// Append the newDir directory to the path, retaining any
// filename that might already be on the path.
//
AnsiString appendDirectory(AnsiString path, AnsiString newDir)
{
  char drillIntoPath[MAXPATH];
  char drive[MAXDRIVE];
  char dir[MAXDIR];
  char file[MAXFILE];
  char ext[MAXEXT];

  fnsplit(path.c_str(),drive,dir,file,ext);

  strcat(dir, newDir.c_str());

  fnmerge(drillIntoPath,drive,dir,file,ext);

  return(AnsiString(drillIntoPath));
}

